
package SOFA.SOFAnode.Util;

/**
 * A Vector-like class for handling arrays of primitive types (int)
 *
 * @author	Vladimir Mencl
 * @version	1.0.0
 */
    public class IntVector {
      public int data[];
      public int size;
      public int used;

      public int sz_initial = 16;

      /**
       * Create an empty IntVector
       */
      public IntVector() { data = null; size=0; used=0; };

      /**
       * Create an IntVector using an int[]
       *
       * @param iarr 	array with values to initialize the IntVector
       * @param docopy	whether a local copy of the array should be created.
       * Otherwise, only a reference to the array is copied
       */
      public IntVector(int iarr[], boolean docopy) { 
	if (docopy) {
	  data=new int[iarr.length];
	  for (int i=0;i<iarr.length;i++) data[i]=iarr[i];
	} else { data=iarr; };
	size=used=iarr.length; 
      };


      /**
       * Create an IntVector by copying the used part of an IntVector
       *
       * @param copyfrom	the IntVector to be used
       */
      public IntVector(IntVector copyfrom) { 
	data=new int[copyfrom.used];
	for (int i=0;i<copyfrom.used;i++) data[i]=copyfrom.data[i];
	size=used=copyfrom.used;
      };

      /**
       * Return a string representation of this IntVector
       *
       * The string is embedded in square brackets, values are separated
       * by commas, no spaces are included.
       *
       * Example: [0,1,4]
       *
       * @returns	the string representation
       */
      public String dump() {
        StringBuffer s = new StringBuffer();
	s.append("[");
	for (int i=0;i<used;i++) { 
	  s.append(data[i]); if (i+1<used) s.append(","); 
	};
	s.append("]");
	return s.toString();
      };

      /**
       * Insert a value into a sorted IntVector, if it is not included yet.
       *
       * The IntVector must be sorted, then it is guaranteed
       * to be sorted afterwards. If the value alread exists in the
       * IntVector, no action is performed.
       *
       * @param	i	the value to insert
       */
      public void sortedAddIntIfNExists(int i) {
	int j=0,t;

	while ((j<used) && (data[j]<i)) j++;

	if (j==used) addInt(i);
	else {
	  if (data[j]==i) return; /* already exists */
	  while (j<used) {
	    t=data[j]; data[j]=i; i=t;
	    j++;
	  };
	  addInt(i);
	};
      };

      /**
       * Create an IntVector containing indexes of elements whose value is 
       * at least minValue
       *
       * The size of the vector is exactly the amount of elements copied 
       * (no further additions are expected)
       *
       * Note: An IntVector created this way is inherently sorted. 
       * (Of course, only untill the first modification).
       *
       * @param iarr 	the array to be used
       * @param minvalue	the minimal value of an element to be 
       * valid (so that it's position is used)
       */
      public IntVector(int iarr[], int minValue) {
	int i,n=0;
	for (i=0;i<iarr.length;i++) if (iarr[i]>=minValue) n++;
	size=n; data = new int[size];
	for (i=0;i<iarr.length;i++) if (iarr[i]>=minValue) data[used++]=i;
      }

      /**
       * add an int to the IntVector
       *
       * checkSize is called to assure there is enough space
       *
       * @param i	value to be added
       * @returns	position of the element inserted
       * @see	#checkSize
       */
      public int addInt(int i) { 
	checkSize(used+1); data[used++]=i; return used-1; 
      };

      public void checkSize(int requested) {
	if (requested>size) {
	  if (size==0) size=sz_initial; else size*=2;
	  int newdata[]=new int[size];
	  for (int i=0;i<used;i++) newdata[i]=data[i];
	  data=newdata;
	};
      };
    };


