/* $Id: Machine.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */

package SOFA.SOFAnode.Util;

import java.util.*;

/**
 * The interface for creating and transforming the finite-state acceptor.
 * The state space is represented by an array of states, for each state 
 * by an array of all action tokens being used. For each pair state,token
 * there is a reference to the IntVector of states being the targets of
 * edges from the state by the transition.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.5.0
 * @see		IntVector
*/
interface Machine {

/**
 * Adds new states to the automaton. Each state is represented by the corresponding
 * representation. Method returns the mapping for transforming indices in 
 * array of added states, to the new state ids.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
 * @param n	an array of new states to be added
 * @param size	length of  added array
*/
    int[] AddStates( IntVector n[][], int size );

/**
 * Reduces the automaton, that all equivalent states are represented as only one state.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
*/
    void Reduce();

/**                 
 * Transforms the automaton to a deterministic one.
 *
 * Optimized by Vladimir Mencl
 *
 * @author      Vladimir Mencl
 * @version     2.0.0
 */                  

    void Determinize();

/**
 * Eliminates all unreachable states.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
*/
    void Reachable();

/**
 * Restrict the automaton by the set passed as the argument.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
 * @param restrict	the set of action tokens to be eliminated
*/
    void Restrict( ActionTokenArray restrict );

/**
 * Restrict the automaton to non-update behavior only.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
*/
    void NonUpdateOnly();

/**
 * Restrict the automaton to update behavior only.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
*/
    void UpdateOnly();

/**
 * Restrict the automaton to update behavior only and remove [UPDATE] events at the end
 * of the traces.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
*/
    void StrippedUpdateOnly();

/**
 * Query the automaton, whether its language is included in the automaton
 * given as the argument. It returns the counterexample in case of failure.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
 * @param m	the automaton to be compared
 * @return	string with counterexample, null for success
*/
    String isIncludedIn( Machine m );

/**
 * Returns the starting state of the automaton.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
 * @return	the starting state
*/
    int Start();

/**
 * Returns the hashtable of accepting states of automaton. The keys and the values
 * are Integer instances with the indices.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
 * @return	the hashtable of states
*/
    Hashtable Stop();

/**
 * Returns the hashtable of update states of automaton. The keys and the values
 * are Integer instances with the indices.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
 * @return	the hashtable of states
*/
    Hashtable Update();

/**
 * Sets a new starting state of the automaton.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
 * @param n	the new starting state
*/
    void SetStart( int n );

/**
 * Adds a new accepting state of the automaton.Returns the target of this edge.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
 * @param n	the new accepting state
*/
    void AddStop( int n );

/**
 * Adds a new state to the automaton. Returns the index of the new state.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
 * @return	the new accepting state
*/
    int addState();

/**
 * Adds a collection of new accepting states of the automaton.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
 * @param n	a list of new accepting states
*/
    void AddStops( Collection n );

/**
 * Sets the accepting states to none.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
*/
    void ClearStops();

/**
 * Adds a new edge.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
 * @param from	index of a state which the edge leads from
 * @param transition	index of an edge identification
 * @param to	index of a state which the edge leads to
*/
    void addEdge( int from, int transition, int to );
    
/**
 * Adds a new edge from the given state to the update state.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
 * @param from	index of a state which the edge leads from
*/
    void addUpdateEdge( int from );

/**
 * Adds all edges of state from to the state to. Effectively merges all edges.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
 * @param from	index of state, which should be enhanced by edges
 * @param to	index of state from which to take the edges
*/
    void Merge( int to, int from  );

/**
 * Checks, whether there is enough state space given as a parameter. In case of
 * failure, it reallocates to the new size. It does not reduce the size.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
 * @param required	required number of states
*/
    void checkSpace( int required );

/**
 * Minimizes the machine to the reduced deterministic form.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
*/
    void Minimize();
}
